#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "PrinterLibs.h"

using namespace std;

int main(int argc, char *argv[])
{
    if(argc > 3 || argc < 2)
    {
        cout << "Usage : program port_name baudrate/tcp port" << endl;
        cout << "Example : ./test /dev/ttyS0 115200" << endl;
        return -1;
    }

    const char *port_name = NULL;
    if(argc >= 2)
    {
        port_name = argv[1];
    }

    int value = 115200;
    if(argc == 3)
    {
        value = atoi(argv[2]);
    }

    // open port 
    void *handle = NULL;
    if(strstr(argv[1],"/dev/tty"))
    {
        handle = Port_OpenCOMIO(port_name, value);
    }else if(strstr(argv[1],"/dev/usb/lp"))
    {
        handle = Port_OpenUSBIO(port_name);
    }else if(strstr(argv[1],"/dev/lp"))
    {
        handle = Port_OpenLPTIO(port_name);
    }else if(strstr(argv[1],"."))
    {
        handle = Port_OpenTCPIO(port_name, value);
    }
    if(handle == NULL)
    {
        cout << "Can not open port. you can use sudo to retry." << endl;
        return -2;
    }

    // set port to Printer
    Port_SetPort(handle);

    // start text printing
    cout << "Start text printing in 3 seconds" << endl;
    sleep(3);

    Pos_Reset();
    Pos_Text("Self Test\n", 1, -2, 1, 1, 0, 0);
    Pos_Text("测试小票\n", 0, -2, 1, 1, 0, 0);
    Pos_Text("測試小票\n", 3, -2, 1, 1, 0, 0);
    Pos_Text("テストシート\n", 4, -2, 1, 1, 0, 0);
    Pos_Text("welcome to use\n", 1, -1, 0, 0, 0, 0x80 | 0x08);
    Pos_Text("欢迎使用我司打印机\n", 0, -2, 0, 0, 0, 0x100);
    Pos_Text("歡迎使用印表機\n", 3, -3, 0, 0, 0, 0x100);
    Pos_Text("ご利用を歓迎する\n", 4, -2, 0, 0, 0, 0x100);
    Pos_FeedLine();
    Pos_Text("Test BarCode\n", 1, -2, 0, 0, 0, 0);
    Pos_Barcode("123456", 0x45, 0, 3, 96, 0, 2);
    Pos_FeedLine();
    Pos_Text("Test QrCode\n", 1, -2, 0, 0, 0, 0);
    Pos_Reset();
    Pos_Align(1);
    Pos_Qrcode("welcome to use\n", 1, 9);
    Pos_FeedLine();
    Pos_Text("Test Double QrCode\n", 1, -2, 0, 0, 0, 0);
    Pos_DoubleQrcode("Self Test", 0, 9, 3, "测试小票", 192, 9, 3, 3);
    Pos_Feed_N_Line(5);
    Pos_FullCutPaper();


    cout << "Start Image printing in 3 seconds" << endl;
    // sleep(3);
    bool ret = Pos_ImagePrint("./TestImage.jpg",384,0);
    if(!ret)
    {
        cout << "print image err" << endl;
    }

    // print Successful
    cout << "Print Successful!!" << endl;
    cout << "Close " << argv[1] << " Port" << endl;
    Port_ClosePort(handle);

    cout << "This is the end of basic operation. Thank you for using our printer" << endl;
    return 0;
}

